--- Fl_lock.cxx.Orig	2007-01-27 20:19:03.000000000 +0100
+++ Fl_lock.cxx	2007-01-27 22:01:01.000000000 +0100
@@ -131,34 +131,15 @@
 #  include <unistd.h>
 #  include <pthread.h>
 
-#  if defined (PTHREAD_MUTEX_RECURSIVE_NP)
-// Linux supports recursive locks, use them directly:
-
-static bool minit;
 static pthread_mutex_t fltk_mutex;
-// this is needed for the Fl_Mutex constructor:
-pthread_mutexattr_t Fl_Mutex_attrib = {PTHREAD_MUTEX_RECURSIVE_NP};
-
-static void lock_function() {
-  if (!minit) {
-    pthread_mutex_init(&fltk_mutex, &Fl_Mutex_attrib);
-    minit = true;
-  }
-  pthread_mutex_lock(&fltk_mutex);
-}
-
-void Fl::unlock() {
-  pthread_mutex_unlock(&fltk_mutex);
-}
-
-#  else
-// Make a recursive lock out of the pthread mutex:
-
-static pthread_mutex_t fltk_mutex = PTHREAD_MUTEX_INITIALIZER;
 static pthread_t owner;
 static int counter;
 
-static void lock_function() {
+static void lock_function_init_std() {
+  pthread_mutex_init(&fltk_mutex, NULL);
+}
+
+static void lock_function_std() {
   if (!counter || owner != pthread_self()) {
     pthread_mutex_lock(&fltk_mutex);
     owner = pthread_self();
@@ -166,10 +147,31 @@
   counter++;
 }
 
-void Fl::unlock() {
+static void unlock_function_std() {
   if (!--counter) pthread_mutex_unlock(&fltk_mutex);
 }
 
+#  ifdef PTHREAD_MUTEX_RECURSIVE
+
+static bool lock_function_init_rec() {
+  pthread_mutexattr_t attrib;
+  pthread_mutexattr_init(&attrib);
+  if (pthread_mutexattr_settype(&attrib, PTHREAD_MUTEX_RECURSIVE)) {
+    pthread_mutexattr_destroy(&attrib);
+    return true;
+  }
+  pthread_mutex_init(&fltk_mutex, &attrib);
+  return false;
+}
+
+static void lock_function_rec() {
+  pthread_mutex_lock(&fltk_mutex);
+}
+
+static void unlock_function_rec() {
+  pthread_mutex_unlock(&fltk_mutex);
+}
+
 #  endif
 
 // Pipe for thread messaging...
@@ -194,14 +196,29 @@
 }
 
 void Fl::lock() {
-  lock_function();
   if (!thread_filedes[1]) { // initialize the mt support
     // Init threads communication pipe to let threads awake FLTK from wait
     pipe(thread_filedes);
     Fl::add_fd(thread_filedes[0], FL_READ, thread_awake_cb);
-    fl_lock_function   = lock_function;
-    fl_unlock_function = Fl::unlock;
+#  ifdef PTHREAD_MUTEX_RECURSIVE
+    if (!lock_function_init_rec()) {
+      fl_lock_function   = lock_function_rec;
+      fl_unlock_function = unlock_function_rec;
+    }
+    else {
+#  endif
+      lock_function_init_std();
+      fl_lock_function   = lock_function_std;
+      fl_unlock_function = unlock_function_std;
+#  ifdef PTHREAD_MUTEX_RECURSIVE
+    }
+#  endif
   }
+  fl_lock_function();
+}
+
+void Fl::unlock() {
+  fl_unlock_function();
 }
 
 void Fl::awake(void* msg) {
